import numpy as np

# AMPD function
def _ampd(sig_input, LSM_limit = 1):
    """Find the peaks in the signal with the AMPD algorithm.
    
        Original implementation by Felix Scholkmann et al. in
        "An Efficient Algorithm for Automatic Peak Detection in 
        Noisy Periodic and Quasi-Periodic Signals", Algorithms 2012,
         5, 588-603

        Parameters
        ----------
        sig_input: ndarray
            The 1D signal given as input to the algorithm
        LSM_limit: float
            Wavelet transform limit as a ratio of full signal length.
            Valid values: 0-1, the LSM array will no longer be calculated after this point
              which results in the inability to find peaks at a scale larger than this factor.
              For example a value of .5 will be unable to find peaks that are of period 
              1/2 * signal length, a default value of 1 will search all LSM sizes.

        Returns
        -------
        pks: ndarray
            The ordered array of peaks found in sig_input
    """
        
    # Create preprocessing linear fit    
    sig_time = np.arange(0, len(sig_input))
    
    # Detrend
    dtr_signal = (sig_input - np.polyval(np.polyfit(sig_time, sig_input, 1), sig_time)).astype(float)
    
    N = len(dtr_signal)
    L = int(np.ceil(N*LSM_limit / 2.0)) - 1
    
    # Generate random matrix
    LSM = np.ones([L, N], dtype='uint8')
    
    # Local minima extraction
    for k in range(1, L):
        LSM[k - 1, np.where((dtr_signal[k:N - k - 1] > dtr_signal[0: N - 2 * k - 1]) & (dtr_signal[k:N - k - 1] > dtr_signal[2 * k: N - 1]))[0]+k] = 0
    
    pks = np.where(np.sum(LSM[0:np.argmin(np.sum(LSM, 1)), :], 0)==0)[0]
    return pks

def get_peak(data, find_button=False, scan_range=1000):
    def AMPD_avoid_MemoryError(data):
        try:
            all_index = _ampd(data)
            all_data = [data[i] for i in all_index]
        except Exception:
            start_index = 0
            is_end = False
            all_index = list()
            all_data = list()

            while not is_end:
                if start_index + scan_range < len(data):
                    end_index = start_index + scan_range
                else:
                    end_index = len(data)
                    is_end = True
                arr_data = data[start_index:end_index]
                arr_index = _ampd(np.array(arr_data))
                for i in range(len(arr_index)):
                    real_index = arr_index[i] + start_index
                    if len(all_index) == 0 or real_index > all_index[-1]:
                        all_index.append(real_index)
                        all_data.append(arr_data[i])
                start_index += arr_index[-1]

        return np.array(all_index), np.array(all_data)

    data = np.array(data)

    if find_button:
        data = -data

    index_peaks, value_peaks = AMPD_avoid_MemoryError(data)

    if find_button:
        return index_peaks, -value_peaks
    else:
        return index_peaks, value_peaks

def _demo():
    print('This is a demo of get_peak.')
    t = np.linspace(-4*np.pi, 4*np.pi, 400)
    noisy_sin_t = np.sin(t) + np.random.normal(0, 1, t.shape) * 0.01
    index_peaks, value_peaks = get_peak(noisy_sin_t)
    index_buttoms, value_buttoms = get_peak(noisy_sin_t, find_button=True)

    import matplotlib.pyplot as plt
    plt.plot(noisy_sin_t, label='sin(t)')
    plt.scatter(index_peaks, value_peaks, label='peak')
    plt.scatter(index_buttoms, value_buttoms, label='buttoms')
    plt.show()

if __name__ == '__main__':
    _demo()